# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-06 02:16
from __future__ import unicode_literals

from django.db import migrations


def position_to_label(apps, schema_editor):
    RelationPartition = apps.get_model("flow", "RelationPartition")
    for partition in RelationPartition.objects.all():
        if partition.position:
            partition.label = partition.position
        partition.save()


class Migration(migrations.Migration):

    dependencies = [
        ("flow", "0015_refactor_relations_1"),
    ]

    operations = [
        migrations.RunPython(position_to_label),
    ]
